@@ -228,11 +228,6 @@ struct in_addr_42 {
 	} S_un_w;
 	afs_uint32 S_addr;
     } S_un;
-#define	s_host	S_un.S_un_b.s_b2	/* host on imp */
-#define	s_net	S_un.S_un_b.s_b1	/* network */
-#define	s_imp	S_un.S_un_w.s_w2	/* imp */
-#define	s_impno	S_un.S_un_b.s_b4	/* imp # */
-#define	s_lh	S_un.S_un_b.s_b3	/* logical host */
 };
 
 #define	mPrintIPAddr(ipaddr)  printf("[%d.%d.%d.%d] ",		\
@@ -307,6 +302,7 @@ int afsd_verbose = 0;		/*Are we being chatty? */
 int afsd_debug = 0;		/*Are we printing debugging info? */
 static int afsd_CloseSynch = 0;	/*Are closes synchronous or not? */
 static int rxmaxmtu = 0;       /* Are we forcing a limit on the mtu? */
+static int rxmaxfrags = 0;      /* Are we forcing a limit on frags? */
 
 #ifdef AFS_SGI62_ENV
 #define AFSD_INO_T ino64_t
@@ -1909,6 +1905,16 @@ mainproc(struct cmd_syndesc *as, void *arock)
 	/* -dynroot-sparse */
 	enable_dynroot = 2;
     }
+    if (as->parms[38].items) {
+	/* -rxmaxfrags */
+	rxmaxfrags = atoi(as->parms[38].items->data);
+    }
+
+    /* parse cacheinfo file if this is a diskcache */
+    if (ParseCacheInfoFile()) {
+	exit(1);
+    }
+
     return 0;
 }
 
@@ -1945,11 +1951,6 @@ afsd_run(void)
 	    printf("%s: My home cell is '%s'\n", rn, LclCellName);
     }
 
-    /* parse cacheinfo file if this is a diskcache */
-    if (ParseCacheInfoFile()) {
-	exit(1);
-    }
-
     if (!enable_nomount) {
 	if (afsd_check_mount(rn, afsd_cacheMountDir)) {
 	    return -1;
@@ -2316,6 +2317,14 @@ afsd_run(void)
 	afsd_call_syscall(AFSOP_CELLINFO, fullpn_CellInfoFile);
     }
 
+    if (rxmaxfrags) {
+	if (afsd_verbose)
+            printf("%s: Setting rxmaxfrags in kernel = %d\n", rn, rxmaxfrags);
+	code = afsd_call_syscall(AFSOP_SET_RXMAXFRAGS, rxmaxfrags);
+        if (code)
+            printf("%s: Error seting rxmaxfrags\n", rn);
+    }
+
     if (rxmaxmtu) {
 	if (afsd_verbose)
             printf("%s: Setting rxmaxmtu in kernel = %d\n", rn, rxmaxmtu);
@@ -2336,7 +2345,7 @@ afsd_run(void)
     if (enable_fakestat) {
 	if (afsd_verbose)
 	    printf("%s: Enabling fakestat support in kernel%s.\n", rn,
-		   (enable_fakestat==2)?" for all mountpoints."
+		   (enable_fakestat==1)?" for all mountpoints."
 		   :" for crosscell mountpoints");
 	code = afsd_call_syscall(AFSOP_SET_FAKESTAT, enable_fakestat);
 	if (code)
@@ -2575,6 +2584,9 @@ afsd_init(void)
     cmd_AddParm(ts, "-rxmaxmtu", CMD_SINGLE, CMD_OPTIONAL, "set rx max MTU to use");
     cmd_AddParm(ts, "-dynroot-sparse", CMD_FLAG, CMD_OPTIONAL,
 		"Enable dynroot support with minimal cell list");
+    cmd_AddParm(ts, "-rxmaxfrags", CMD_SINGLE, CMD_OPTIONAL,
+                "Set the maximum number of UDP fragments Rx should send/receive"
+                " per Rx packet");
 }
 
 int
